@@ -1,5 +1,5 @@
 #' @name dispRity.metric
-#' @aliases dimension.level3.fun dimension.level2.fun dimension.level1.fun variances ranges centroids mode.val ellipse.volume convhull.surface convhull.volume diagonal ancestral.dist pairwise.dist span.tree.length n.ball.volume radius neighbours displacements quantiles func.eve func.div
+#' @aliases dimension.level3.fun dimension.level2.fun dimension.level1.fun variances ranges centroids mode.val ellipse.volume convhull.surface convhull.volume diagonal ancestral.dist pairwise.dist span.tree.length n.ball.volume radius neighbours displacements quantiles func.eve func.div angles deviations
 #' @title Disparity metrics
 #'
 #' @description Different implemented disparity metrics.
@@ -34,9 +34,9 @@
 #'          \item WARNING: this function assumes that the input matrix is ordinated and calculates the matrix' eigen values from the matrix as \code{abs(apply(var(matrix),2, sum))} (which is equivalent to \code{eigen(var(matrix))$values} but faster). These values are the correct eigen values for any matrix but differ from the ones output from \code{\link[stats]{cmdscale}} and \code{\link[ape]{pcoa}} because these later have their eigen values multiplied by the number of elements - 1 (i.e. \code{abs(apply(var(matrix),2, sum)) * nrow(matrix) -1 }). Specific eigen values can always be provided manually through \code{ellipse.volume(matrix, eigen.value = my_val)} (or \code{dispRity(matrix, metric = ellipse.volume, eigen.value = my_val)}).
 #'      }
 #' 
-#'   \item \code{func.div}: The functional divergence (Vill'{e}ger et al. 2008): the ratio of deviation from the centroid (this is similar to \code{\link[FD]{dbFD}$FDiv}).
+#'   \item \code{func.div}: The functional divergence (Vill'{e}ger et al. 2008): the ratio of deviation from the centroid (this is similar to \code{FD::dbFD$FDiv}).
 #' 
-#'   \item \code{func.eve}: The functional evenness (Vill'{e}ger et al. 2008): the minimal spanning tree distances evenness (this is similar to \code{\link[FD]{dbFD}$FEve}). If the matrix used is not a distance matrix, the distance method can be passed using, for example \code{method = "euclidean"} (default).
+#'   \item \code{func.eve}: The functional evenness (Vill'{e}ger et al. 2008): the minimal spanning tree distances evenness (this is similar to \code{FD::dbFD$FEve}). If the matrix used is not a distance matrix, the distance method can be passed using, for example \code{method = "euclidean"} (default).
 #' 
 #'   \item \code{mode.val}: calculates the modal value of a vector.
 #'
@@ -50,9 +50,12 @@
 #' The currently implemented dimension-level 2 metrics are:
 #' \itemize{
 #'   \item \code{ancestral.dist}: calculates the distance between each tip and node and their ancestral. This function needs either (1) \code{matrix}/\code{list} from \code{\link{nodes.coordinates}}; or a \code{tree} (\code{"phylo"}) and \code{full} (\code{"logical"}) argument to calculate the node coordinates for the direct descendants (\code{full = FALSE}) or all descendants down to the root (\code{full = TRUE}). NOTE: distance is calculated as \code{"euclidean"} by default, this can be changed using the \code{method} argument.
-#'
+#' 
+#'   \item \code{angles}: calculates the angles of the main axis of variation per dimension in a \code{matrix}. The angles are calculated using the least square algorithm from the \code{\link[stats]{lm}} function. The unit of the angle can be changed through the \code{unit} argument (either \code{"degree"} (default), \code{radian} or \code{slope}) and a base angle to measure the angle from can be passed through the \code{base} argument (by default \code{base = 0}, measuring the angle from the horizontal line (not that the \code{base} argument has to be passed in the same unit as \code{unit}). When estimating the slope through \code{\link[stats]{lm}}, you can use the option \code{significant} to only consider significant slopes (\code{TRUE}) or not (\code{FALSE} - default).
+#' 
 #'   \item \code{centroids}: calculates the distance between each row and the centroid of the matrix (Lalibert'{e} 2010). This function can take an optional arguments \code{centroid} for defining the centroid (if missing (default), the centroid of the matrix is used). This argument can be either a subset of coordinates matching the matrix's dimensions (e.g. \code{c(0, 1, 2)} for a matrix with three columns) or a single value to be the coordinates of the centroid (e.g. \code{centroid = 0} will set the centroid coordinates to \code{c(0, 0, 0)} for a three dimensional matrix). NOTE: distance is calculated as \code{"euclidean"} by default, this can be changed using the \code{method} argument.
 #'
+#' \item \code{deviations}: calculates the minimal Euclidean distance between each element in and the hyperplane (or line if 2D, or a plane if 3D). You can specify equation of hyperplane of \emph{d} dimensions in the \eqn{intercept + ax + by + ... + nd = 0} format. For example the line \eqn{y = 3x + 1} should be entered as \code{c(1, 3, -1)} or the plane \eqn{x + 2y - 3z = 44} as \code{c(44, 1, 2, -3,)}. If missing the \code{hyperplane} (default) is calculated using a least square regression using a gaussian \code{\link[stats]{glm}}. Extract arguments can be passed to \code{\link[stats]{glm}} through \code{...}. When estimating the hyperplane, you can use the option \code{significant} to only consider significant slopes (\code{TRUE}) or not (\code{FALSE} - default).
 #'   \item \code{displacements}: calculates the ratio between the distance to the centroid (see \code{centroids} above) and the distance from a reference (by default the origin of the space). The reference can be changed through the \code{reference} argument. NOTE: distance is calculated as \code{"euclidean"} by default, this can be changed using the \code{method} argument.
 #'
 #'   \item \code{neighbours}: calculates the distance to a neighbour (Foote 1990). By default this is the distance to the nearest neighbour (\code{which = min}) but can be set to any dimension level - 1 function (e.g. \code{which = mean} gives the distance to the most average neighbour). NOTE: distance is calculated as \code{"euclidean"} by default, this can be changed using the \code{method} argument. 
@@ -104,6 +107,12 @@
 #' ancestral.dist(dummy_matrix, nodes.coords = direct_anc_centroids)
 #' ## Calculating the distances from all the ancestral nodes
 #' ancestral.dist(dummy_matrix, nodes.coords = all_anc_centroids)
+#' 
+#' ## angles
+#' ## The angles in degrees of each axis
+#' angles(dummy_matrix)
+#' ## The angles in slope from the 1:1 slope (Beta = 1)
+#' angles(dummy_matrix, unit = "slope", base = 1)
 #'
 #' ## centroids
 #' ## Distances between each row and centroid of the matrix
@@ -112,6 +121,12 @@
 #' centroids(dummy_matrix, centroid = c(1,2,3,4,5,6,7,8,9,10))
 #' ## Distances between each row and the origin
 #' centroids(dummy_matrix, centroid = 0)
+#' 
+#' ## deviations
+#' ## The deviations from the least square hyperplane
+#' deviations(dummy_matrix)
+#' ## The deviations from the plane between the x and y axis
+#' deviations(dummy_matrix, hyperplane = c(0,1,1,0,0,0,0,0,0,0,0))
 #'
 #' ## diagonal
 #' ## Matrix diagonal
@@ -215,7 +230,9 @@ dimension.level3.fun <- function(matrix, ...) {
 dimension.level2.fun <- function(matrix, ...) {
     cat("Dimension level 2 functions implemented in dispRity:\n")
     cat("?ancestral.dist\n")
+    cat("?angles\n")
     cat("?centroids\n")
+    cat("?deviations\n")
     cat("?displacements\n")
     cat("?neighbours\n")
     cat("?pairwise.dist\n")
@@ -505,6 +522,106 @@ func.div <- function(matrix) {
     return((sum(dist_centroid) - mean_dis_cent * (obs-1)) / ((sum(abs(dist_centroid - mean_dis_cent) + dist_centroid))/obs))
 }
 
+## Angles measurements
+angles <- function(matrix, unit = "degree", base = 0, significant = FALSE) {
+
+    ## Check the unit
+    all_methods <- c("degree", "radian", "slope")
+    check.method(unit, all_methods, "Angle unit")
+
+    ## Check the base
+    check.class(base, c("numeric", "integer"))
+
+    ## Generate the base angle (slope/radian/angle = 0)
+    base_angle <- seq_len(nrow(matrix))
+
+    ## Select the right slope function
+    get.slope.significant <- function(X, base_angle) {
+        model <- lm(base_angle ~ X)
+        return(ifelse(summary(model)[[4]][[8]] < 0.05, model$coefficients[[2]], 0))
+    }
+    get.slope.nonsignificant <- function(X, base_angle) {
+        lm(base_angle ~ X)$coefficients[[2]]
+    }
+    if(significant) {
+        get.slope <- get.slope.significant
+    } else {
+        get.slope <- get.slope.nonsignificant
+    }
+
+    ## Get all the slopes
+    slopes <- apply(matrix, 2, get.slope, base_angle = base_angle)
+
+    ## Convert the slopes
+    angles <- switch(unit,
+        #degree = {atan(slopes/(1 + base_slope * (base_slope + slopes))) * 180/pi},
+        degree = {atan(slopes) * 180/pi},
+        radian =  {atan(slopes)},
+        # radian = {atan(slopes/(1 + base_slope * (base_slope + slopes)))},
+        slope  = {slopes}
+        )
+
+    ## Add a base angle (if not 0)
+    if(base != 0) {
+        return(angles + base)
+    } else {
+        return(angles)
+    }
+}
+
+## Deviations
+deviations <- function(matrix, hyperplane, ..., significant = FALSE) {
+
+    ## Get the dimensions
+    dimensions <- ncol(matrix)
+
+    if(missing(hyperplane)) {
+        ## If the data is unidimensional
+        if(ncol(matrix) == 1) {
+            data <- as.data.frame(cbind(seq_along(1:nrow(matrix)), matrix))
+        } else {
+            data <- as.data.frame(matrix)
+        }
+        colnames(data) <- paste0("c", seq_along(1:ncol(data)))
+
+        ## Calculate the hyperplane
+        formula <- "c1 ~ c2"
+        if(ncol(matrix) > 2) {
+            formula <- paste0(formula, " + ", paste(colnames(data)[-c(1,2)], collapse = " + "))
+        }
+
+        ## Get the regression coefficients
+        if(!significant) {
+            equation <- glm(formula = formula, data = data, ...)$coefficients
+            ## Replace NAs by zeros
+            equation <- ifelse(is.na(equation), 0, equation)
+        } else {
+            ## Run the model
+            model <- glm(formula = formula, data = data, ...)
+            ## Check the coefficients p values
+            equation <- ifelse(is.na(model$coefficients), 0, model$coefficients)
+            ## Check p_values
+            p_values <- which(ifelse(is.na(summary(model)$coefficients[,4]), 0, summary(model)$coefficients[,4]) > 0.5)
+            ## Get the hyperplane definition
+            if(length(p_values) != 0) {
+                equation[p_values] <- 0
+            }
+        }
+        ## Correct for the hyperplane equation to be equal to 0
+        hyperplane <- c(equation[1], -1, equation[-1])
+    } else {
+        check.length(hyperplane, dimensions+1, paste0(" must be of length ", dimensions, "+1."))
+    }
+    ## Distance function
+    distance <- function(point, hyperplane, dimensions) {
+        ## Get the number of dimensions
+        return(abs(sum(point*hyperplane[2:(dimensions+1)], hyperplane[1]))/sqrt(sum(hyperplane[2:(dimensions+1)]^2)))
+    }
+
+    ## Get all distance
+    return(apply(matrix, 1, FUN = distance, hyperplane, dimensions))
+}
+
 
 #' @title Nodes coordinates
 #'
